package com.fzm.admin.controller;

import com.fzm.common.annotation.Authentication;
import com.fzm.common.entity.Nft;
import com.fzm.common.entity.vo.NftListVo;
import com.fzm.common.model.ResponseModel;
import com.fzm.common.service.NftService;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

/**
 * @author tangtuo
 * @date 2021/7/5 15:34
 */
@Authentication
@RestController
@RequestMapping("/nft")
@Api(tags = "剧本nft管理")
public class NftController {

    @Resource
    private NftService nftService;

    @GetMapping("/list")
    @ApiOperation(value = "查询nft列表")
    public ResponseModel<PageInfo<NftListVo>> list(@ApiParam(value = "页码", required = true) @RequestParam Integer pageNum,
                                                   @ApiParam(value = "每页记录数", required = true) @RequestParam Integer pageSize,
                                                   @ApiParam(value = "类目id") @RequestParam(required = false) Integer categoryId,
                                                   @ApiParam(value = "作品名称") @RequestParam(required = false) String name,
                                                   @ApiParam(value = "作品标签") @RequestParam(required = false) String theme,
                                                   @ApiParam(value = "注册手机号") @RequestParam(required = false) String telephone,
                                                   @ApiParam(value = "上架状态 0-未上架  1-已上架") @RequestParam(required = false) Integer status,
                                                   @ApiParam(value = "创建开始日期,yyyy-MM-dd格式") @RequestParam(required = false) String start,
                                                   @ApiParam(value = "创建截止日期,yyyy-MM-dd格式") @RequestParam(required = false) String end) {
        PageInfo<NftListVo> page = nftService.page(pageNum, pageSize, categoryId, name, theme, telephone, status, start, end);
        return ResponseModel.success(page);
    }

    @ApiOperation(value = "置顶")
    @PostMapping("/top/{id}")
    public ResponseModel<Boolean> top(@ApiParam(value = "nft主键") @PathVariable Integer id) {
        return ResponseModel.success(nftService.top(id));
    }

    @ApiOperation(value = "取消置顶")
    @PostMapping("/untop/{id}")
    public ResponseModel<Boolean> untop(@ApiParam(value = "nft主键") @PathVariable Integer id) {
        return ResponseModel.success(nftService.untop(id));
    }

    @ApiOperation(value = "上架")
    @PostMapping("/onShelf/{id}")
    public ResponseModel onShelf(@ApiParam(value = "nft主键") @PathVariable Integer id) {
        return ResponseModel.success(nftService.onShelf(id));
    }

    @ApiOperation(value = "下架")
    @PostMapping("/offShelf/{id}")
    public ResponseModel offShelf(@ApiParam(value = "nft主键") @PathVariable Integer id) {
        return ResponseModel.success(nftService.offShelf(id));
    }

    @ApiOperation(value = "查看详情")
    @PostMapping("/detail/{id}")
    public ResponseModel<Nft> getById(@ApiParam(value = "nft主键") @PathVariable Integer id) {
        return ResponseModel.success(nftService.getById(id));
    }

    @ApiOperation(value = "下载文件")
    @PostMapping("/download/{id}")
    public void download(@ApiParam(value = "nft主键") @PathVariable Integer id) {
        nftService.download(id);
    }
}
