package com.fzm.admin.controller;

import com.fzm.common.annotation.Authentication;
import com.fzm.common.entity.vo.UserListVo;
import com.fzm.common.entity.vo.UserStatisticVo;
import com.fzm.common.model.ResponseModel;
import com.fzm.common.service.UserService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author tangtuo
 * @date 2021/7/5 10:01
 */
@Authentication
@RestController
@RequestMapping("/admin")
@Api(tags = "用户管理")
public class UserController {

    @Resource
    private UserService userService;

    @GetMapping("statistic")
    @ApiOperation(value = "用户概览", notes = "获取已注册用户、已实名用户等统计信息")
    public ResponseModel<UserStatisticVo> getStatistic() {
        UserStatisticVo statisticVo = userService.getStatistic();
        return ResponseModel.success(statisticVo);
    }

    @GetMapping("/list")
    @ApiOperation(value = "用户列表")
    public ResponseModel<PageInfo<UserListVo>> list(@ApiParam(value = "页码", required = true) @RequestParam Integer pageNum,
                                                    @ApiParam(value = "每页记录数", required = true) @RequestParam Integer pageSize,
                                                    @ApiParam(value = "手机号") @RequestParam(required = false) String telephone,
                                                    @ApiParam(value = "昵称") @RequestParam(required = false) String nickname,
                                                    @ApiParam(value = "认证类型  0-个人认证  1-企业认证") @RequestParam(required = false) Integer authType,
                                                    @ApiParam(value = "注册开始日期,yyyy-MM-dd格式") @RequestParam(required = false) String start,
                                                    @ApiParam(value = "注册截止日期,yyyy-MM-dd格式") @RequestParam(required = false) String end) {
        PageHelper.startPage(pageNum, pageSize);
        List<UserListVo> list = userService.list(telephone, nickname, authType, start, end);
        return ResponseModel.success(new PageInfo<>(list));
    }
}
