package com.fzm.common.service.impl;

import cn.fzm.chain.simplesdk.client.ParaChainClient;
import cn.fzm.chain.simplesdk.constant.EvmTransferEnum;
import cn.fzm.chain.simplesdk.constant.TxStatusEnum;
import cn.fzm.chain.simplesdk.model.TxResult;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.CharsetUtil;
import cn.hutool.core.util.HexUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fzm.common.constant.RedisConstant;
import com.fzm.common.constant.SystemConstant;
import com.fzm.common.entity.*;
import com.fzm.common.entity.dto.ChainDto;
import com.fzm.common.entity.vo.CollectionNftVo;
import com.fzm.common.entity.vo.NftListVo;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.mapper.NftMapper;
import com.fzm.common.params.NftTransferParam;
import com.fzm.common.properties.SmsProperties;
import com.fzm.common.service.*;
import com.fzm.common.utils.JwtUtil;
import com.fzm.common.utils.OssUtil;
import com.fzm.common.utils.RedisUtil;
import com.fzm.common.utils.SmsUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author tangtuo
 * @date 2021/6/30 15:55
 */
@Slf4j
@Service
@Transactional(rollbackFor = RuntimeException.class)
public class NftServiceImpl extends ServiceImpl<NftMapper, Nft> implements NftService {

    @Resource
    private ParaChainClient paraChainClient;

    @Resource
    RedisUtil redisUtil;

    @Resource
    private UserService userService;

    @Resource
    private NftMapper nftMapper;

    @Resource
    private CategoryService categoryService;

    @Resource
    private CollectionService collectionService;

    @Resource
    private NftTransferRecordService nftTransferRecordService;

    @Resource
    private OssUtil ossUtil;

    @Resource
    private SmsUtil smsUtil;

    @Resource
    private SmsProperties smsProperties;

    @Resource
    private HttpServletRequest request;

    @Value("${chain.para.contract-name}")
    private String contractName;

    @Value("${chain.para.contract-address}")
    private String contractAddr;

    @Override
    public NftDto saveNft(Nft nft) {
        Integer userId = JwtUtil.getUserIdFromToken(request.getHeader("Authorization"));
        nft.setUserId(userId);
        save(nft);
        User user = userService.getById(userId);
        NftDto nftDto = new NftDto();
        // 获取用户的钱包地址
        String wallet = user.getWallet();
        nftDto.setNftId(generateNftId(nft.getCategoryId()));
        nftDto.setWallet(wallet);
        nftDto.setId(nft.getId());
        nftDto.setFileHash(nft.getFileHash());
        return nftDto;
    }

    @Override
    public Integer publish(NftDto nftDto) {
        User user = userService.getUserByToken();
        String wallet = user.getWallet();
        // 获取用户的私钥
        String privkey = paraChainClient.walletDumpPrivkey(wallet);
        // 生产tokenId
        Snowflake snowflake = IdUtil.getSnowflake(1, 1);
        long tokenId = snowflake.nextId();
        Nft nft = getById(nftDto.getId());
        Map<String, String> map = new HashMap<>();
        map.put("hash", nftDto.getFileHash());
        map.put("publishAddress", wallet);
        map.put("author", nft.getAuthor());
        String str = JSONUtil.toJsonStr(map);
        String tokenInfo = HexUtil.encodeHexStr(str, CharsetUtil.CHARSET_UTF_8);
        log.info("tokenInfo:{}", tokenInfo);
        // 发行nft
        String abi = "[{\"inputs\":[],\"stateMutability\":\"nonpayable\",\"type\":\"constructor\"},{\"anonymous\":false,\"inputs\":[{\"indexed\":true,\"internalType\":\"address\",\"name\":\"account\",\"type\":\"address\"},{\"indexed\":true,\"internalType\":\"address\",\"name\":\"operator\",\"type\":\"address\"},{\"indexed\":false,\"internalType\":\"bool\",\"name\":\"approved\",\"type\":\"bool\"}],\"name\":\"ApprovalForAll\",\"type\":\"event\"},{\"anonymous\":false,\"inputs\":[{\"indexed\":true,\"internalType\":\"address\",\"name\":\"operator\",\"type\":\"address\"},{\"indexed\":true,\"internalType\":\"address\",\"name\":\"from\",\"type\":\"address\"},{\"indexed\":true,\"internalType\":\"address\",\"name\":\"to\",\"type\":\"address\"},{\"indexed\":false,\"internalType\":\"uint256[]\",\"name\":\"ids\",\"type\":\"uint256[]\"},{\"indexed\":false,\"internalType\":\"uint256[]\",\"name\":\"values\",\"type\":\"uint256[]\"}],\"name\":\"TransferBatch\",\"type\":\"event\"},{\"anonymous\":false,\"inputs\":[{\"indexed\":true,\"internalType\":\"address\",\"name\":\"operator\",\"type\":\"address\"},{\"indexed\":true,\"internalType\":\"address\",\"name\":\"from\",\"type\":\"address\"},{\"indexed\":true,\"internalType\":\"address\",\"name\":\"to\",\"type\":\"address\"},{\"indexed\":false,\"internalType\":\"uint256\",\"name\":\"id\",\"type\":\"uint256\"},{\"indexed\":false,\"internalType\":\"uint256\",\"name\":\"value\",\"type\":\"uint256\"}],\"name\":\"TransferSingle\",\"type\":\"event\"},{\"anonymous\":false,\"inputs\":[{\"indexed\":false,\"internalType\":\"string\",\"name\":\"value\",\"type\":\"string\"},{\"indexed\":true,\"internalType\":\"uint256\",\"name\":\"id\",\"type\":\"uint256\"}],\"name\":\"URI\",\"type\":\"event\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"account\",\"type\":\"address\"},{\"internalType\":\"uint256\",\"name\":\"id\",\"type\":\"uint256\"}],\"name\":\"balanceOf\",\"outputs\":[{\"internalType\":\"uint256\",\"name\":\"\",\"type\":\"uint256\"}],\"stateMutability\":\"view\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address[]\",\"name\":\"accounts\",\"type\":\"address[]\"},{\"internalType\":\"uint256[]\",\"name\":\"ids\",\"type\":\"uint256[]\"}],\"name\":\"balanceOfBatch\",\"outputs\":[{\"internalType\":\"uint256[]\",\"name\":\"\",\"type\":\"uint256[]\"}],\"stateMutability\":\"view\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"uint256\",\"name\":\"tokenId\",\"type\":\"uint256\"}],\"name\":\"getTokenInfo\",\"outputs\":[{\"internalType\":\"string\",\"name\":\"\",\"type\":\"string\"}],\"stateMutability\":\"view\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"account\",\"type\":\"address\"},{\"internalType\":\"address\",\"name\":\"operator\",\"type\":\"address\"}],\"name\":\"isApprovedForAll\",\"outputs\":[{\"internalType\":\"bool\",\"name\":\"\",\"type\":\"bool\"}],\"stateMutability\":\"view\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"owner\",\"type\":\"address\"},{\"internalType\":\"uint256\",\"name\":\"tokenId\",\"type\":\"uint256\"},{\"internalType\":\"uint256\",\"name\":\"amount\",\"type\":\"uint256\"}],\"name\":\"mintToken\",\"outputs\":[],\"stateMutability\":\"nonpayable\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"from\",\"type\":\"address\"},{\"internalType\":\"address\",\"name\":\"to\",\"type\":\"address\"},{\"internalType\":\"uint256[]\",\"name\":\"ids\",\"type\":\"uint256[]\"},{\"internalType\":\"uint256[]\",\"name\":\"amounts\",\"type\":\"uint256[]\"},{\"internalType\":\"bytes\",\"name\":\"data\",\"type\":\"bytes\"}],\"name\":\"safeBatchTransferFrom\",\"outputs\":[],\"stateMutability\":\"nonpayable\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"from\",\"type\":\"address\"},{\"internalType\":\"address\",\"name\":\"to\",\"type\":\"address\"},{\"internalType\":\"uint256\",\"name\":\"id\",\"type\":\"uint256\"},{\"internalType\":\"uint256\",\"name\":\"amount\",\"type\":\"uint256\"},{\"internalType\":\"bytes\",\"name\":\"data\",\"type\":\"bytes\"}],\"name\":\"safeTransferFrom\",\"outputs\":[],\"stateMutability\":\"nonpayable\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"operator\",\"type\":\"address\"},{\"internalType\":\"bool\",\"name\":\"approved\",\"type\":\"bool\"}],\"name\":\"setApprovalForAll\",\"outputs\":[],\"stateMutability\":\"nonpayable\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"uint256\",\"name\":\"tokenId\",\"type\":\"uint256\"},{\"internalType\":\"string\",\"name\":\"tokenInfo\",\"type\":\"string\"}],\"name\":\"setTokenInfo\",\"outputs\":[],\"stateMutability\":\"nonpayable\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"bytes4\",\"name\":\"interfaceId\",\"type\":\"bytes4\"}],\"name\":\"supportsInterface\",\"outputs\":[{\"internalType\":\"bool\",\"name\":\"\",\"type\":\"bool\"}],\"stateMutability\":\"view\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"uint256\",\"name\":\"\",\"type\":\"uint256\"}],\"name\":\"uri\",\"outputs\":[{\"internalType\":\"string\",\"name\":\"\",\"type\":\"string\"}],\"stateMutability\":\"view\",\"type\":\"function\"}]";
        String hash = paraChainClient.evmPublishToken(abi, contractAddr, EvmTransferEnum.ERC1155, 1000000L, wallet, privkey, tokenId, 1, str, true);
        if (StringUtils.isBlank(hash)) {
            throw GlobalException.newException(ResultCode.FAILED, "nft发行失败");
        }
        String tradeHash = paraChainClient.evmSetTokenInfoNew(abi, contractAddr, 1000000L, tokenId, tokenInfo, wallet, privkey, true);
        if (StringUtils.isBlank(tradeHash)) {
            throw GlobalException.newException(ResultCode.PUBLISH_ERROR, "nft发行失败");
        }
        // 确认交易结果
        TxResult txResult = paraChainClient.cycleConfirmTxWithHash(hash, true, 1000);
        if (!TxStatusEnum.SUCCESS.equals(txResult.getStatus())) {
            throw GlobalException.newException(ResultCode.PUBLISH_ERROR, txResult.getErrMsg().getValue());
        }

        String realHash = paraChainClient.getRealTxHashFromGrp(hash);
        /*TxResult txResult = paraChainClient.cycleConfirmTxWithHash(realHash, false, 1000);
        if (!TxStatusEnum.SUCCESS.equals(txResult.getStatus())) {
            throw GlobalException.newException(ResultCode.PUBLISH_ERROR, txResult.getErrMsg().getValue());
        }*/
        nft.setNftHash(realHash);
        nft.setTokenId(tokenId);
        nft.setPublishTime(new Date());
        nft.setNftId(nftDto.getNftId());
        nft.setPublishAddress(wallet);
        updateById(nft);
        // 如果用户是第一次发行作品,把用户的isPublish修改成1,并清空用户统计的缓存信息
        if (SystemConstant.BOOLEAN_DATA_FALSE.equals(user.getIsPublish())) {
            User u = new User().setId(user.getId()).setIsPublish(SystemConstant.BOOLEAN_DATA_TRUE);
            userService.updateById(u);
            redisUtil.delete("user::statistic");
        }
        return nft.getId();
    }

    @Override
    public Boolean transfer(NftTransferParam param) {
        Nft nft = getById(param.getId());
        if (nft == null) {
            throw GlobalException.newException(ResultCode.DATA_ERROR, "没找到此nft详情");
        }
        // 转出人的个人信息
        User user = userService.getUserByToken();
        if (!nft.getUserId().equals(user.getId())) {
            throw GlobalException.newException(ResultCode.TRANSFER_ERROR, "您无权转让他人的nft");
        }
        // 校验短信验证码
        if (!smsUtil.validateCode(getCodeType(param.getCodeType()), user.getTelephone(), param.getCode(), param.getCodeType())) {
            throw GlobalException.newException(ResultCode.TRANSFER_ERROR, "短信验证码校验失败");
        }
        // 校验接收人的个人信息
        User receiveUser = userService.getUserByWallet(param.getReceiveWallet());
        if (receiveUser == null) {
            throw GlobalException.newException(ResultCode.TRANSFER_ERROR, "接手人非本系统注册用户,暂不支持转让");
        }
        // 不能自己向自己转让nft
        if (user.getWallet().equals(receiveUser.getWallet())) {
            throw GlobalException.newException(ResultCode.TRANSFER_ERROR, "不能向自己转让nft");
        }
        String abi = "[{\"inputs\":[],\"stateMutability\":\"nonpayable\",\"type\":\"constructor\"},{\"anonymous\":false,\"inputs\":[{\"indexed\":true,\"internalType\":\"address\",\"name\":\"account\",\"type\":\"address\"},{\"indexed\":true,\"internalType\":\"address\",\"name\":\"operator\",\"type\":\"address\"},{\"indexed\":false,\"internalType\":\"bool\",\"name\":\"approved\",\"type\":\"bool\"}],\"name\":\"ApprovalForAll\",\"type\":\"event\"},{\"anonymous\":false,\"inputs\":[{\"indexed\":true,\"internalType\":\"address\",\"name\":\"operator\",\"type\":\"address\"},{\"indexed\":true,\"internalType\":\"address\",\"name\":\"from\",\"type\":\"address\"},{\"indexed\":true,\"internalType\":\"address\",\"name\":\"to\",\"type\":\"address\"},{\"indexed\":false,\"internalType\":\"uint256[]\",\"name\":\"ids\",\"type\":\"uint256[]\"},{\"indexed\":false,\"internalType\":\"uint256[]\",\"name\":\"values\",\"type\":\"uint256[]\"}],\"name\":\"TransferBatch\",\"type\":\"event\"},{\"anonymous\":false,\"inputs\":[{\"indexed\":true,\"internalType\":\"address\",\"name\":\"operator\",\"type\":\"address\"},{\"indexed\":true,\"internalType\":\"address\",\"name\":\"from\",\"type\":\"address\"},{\"indexed\":true,\"internalType\":\"address\",\"name\":\"to\",\"type\":\"address\"},{\"indexed\":false,\"internalType\":\"uint256\",\"name\":\"id\",\"type\":\"uint256\"},{\"indexed\":false,\"internalType\":\"uint256\",\"name\":\"value\",\"type\":\"uint256\"}],\"name\":\"TransferSingle\",\"type\":\"event\"},{\"anonymous\":false,\"inputs\":[{\"indexed\":false,\"internalType\":\"string\",\"name\":\"value\",\"type\":\"string\"},{\"indexed\":true,\"internalType\":\"uint256\",\"name\":\"id\",\"type\":\"uint256\"}],\"name\":\"URI\",\"type\":\"event\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"account\",\"type\":\"address\"},{\"internalType\":\"uint256\",\"name\":\"id\",\"type\":\"uint256\"}],\"name\":\"balanceOf\",\"outputs\":[{\"internalType\":\"uint256\",\"name\":\"\",\"type\":\"uint256\"}],\"stateMutability\":\"view\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address[]\",\"name\":\"accounts\",\"type\":\"address[]\"},{\"internalType\":\"uint256[]\",\"name\":\"ids\",\"type\":\"uint256[]\"}],\"name\":\"balanceOfBatch\",\"outputs\":[{\"internalType\":\"uint256[]\",\"name\":\"\",\"type\":\"uint256[]\"}],\"stateMutability\":\"view\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"uint256\",\"name\":\"tokenId\",\"type\":\"uint256\"}],\"name\":\"getTokenInfo\",\"outputs\":[{\"internalType\":\"string\",\"name\":\"\",\"type\":\"string\"}],\"stateMutability\":\"view\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"account\",\"type\":\"address\"},{\"internalType\":\"address\",\"name\":\"operator\",\"type\":\"address\"}],\"name\":\"isApprovedForAll\",\"outputs\":[{\"internalType\":\"bool\",\"name\":\"\",\"type\":\"bool\"}],\"stateMutability\":\"view\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"owner\",\"type\":\"address\"},{\"internalType\":\"uint256\",\"name\":\"tokenId\",\"type\":\"uint256\"},{\"internalType\":\"uint256\",\"name\":\"amount\",\"type\":\"uint256\"}],\"name\":\"mintToken\",\"outputs\":[],\"stateMutability\":\"nonpayable\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"from\",\"type\":\"address\"},{\"internalType\":\"address\",\"name\":\"to\",\"type\":\"address\"},{\"internalType\":\"uint256[]\",\"name\":\"ids\",\"type\":\"uint256[]\"},{\"internalType\":\"uint256[]\",\"name\":\"amounts\",\"type\":\"uint256[]\"},{\"internalType\":\"bytes\",\"name\":\"data\",\"type\":\"bytes\"}],\"name\":\"safeBatchTransferFrom\",\"outputs\":[],\"stateMutability\":\"nonpayable\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"from\",\"type\":\"address\"},{\"internalType\":\"address\",\"name\":\"to\",\"type\":\"address\"},{\"internalType\":\"uint256\",\"name\":\"id\",\"type\":\"uint256\"},{\"internalType\":\"uint256\",\"name\":\"amount\",\"type\":\"uint256\"},{\"internalType\":\"bytes\",\"name\":\"data\",\"type\":\"bytes\"}],\"name\":\"safeTransferFrom\",\"outputs\":[],\"stateMutability\":\"nonpayable\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"operator\",\"type\":\"address\"},{\"internalType\":\"bool\",\"name\":\"approved\",\"type\":\"bool\"}],\"name\":\"setApprovalForAll\",\"outputs\":[],\"stateMutability\":\"nonpayable\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"uint256\",\"name\":\"tokenId\",\"type\":\"uint256\"},{\"internalType\":\"string\",\"name\":\"tokenInfo\",\"type\":\"string\"}],\"name\":\"setTokenInfo\",\"outputs\":[],\"stateMutability\":\"nonpayable\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"bytes4\",\"name\":\"interfaceId\",\"type\":\"bytes4\"}],\"name\":\"supportsInterface\",\"outputs\":[{\"internalType\":\"bool\",\"name\":\"\",\"type\":\"bool\"}],\"stateMutability\":\"view\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"uint256\",\"name\":\"\",\"type\":\"uint256\"}],\"name\":\"uri\",\"outputs\":[{\"internalType\":\"string\",\"name\":\"\",\"type\":\"string\"}],\"stateMutability\":\"view\",\"type\":\"function\"}]";
        String tokenInfo = paraChainClient.evmGetTokenInfoNew(abi, contractAddr, nft.getTokenId(), user.getWallet());
        // 转让nft
        String hash = paraChainClient.evmTransferNew(abi, contractAddr, EvmTransferEnum.ERC1155, 1000000L, param.getReceiveWallet(), nft.getTokenId(), param.getCount(), tokenInfo, user.getWallet(), null, true);
        if (StringUtils.isBlank(hash)) {
            throw GlobalException.newException(ResultCode.TRANSFER_ERROR, "nft转让失败");
        }
        // 确认转让结果
        TxResult txResult = paraChainClient.cycleConfirmTxWithHash(hash, true, 1000);
        if (!TxStatusEnum.SUCCESS.equals(txResult.getStatus())) {
            throw GlobalException.newException(ResultCode.TRANSFER_ERROR, txResult.getErrMsg().getValue());
        }
        String realHash = paraChainClient.getRealTxHashFromGrp(hash);
        if (StringUtils.isBlank(realHash)) {
            throw GlobalException.newException(ResultCode.TRANSFER_ERROR);
        }
        // 修改nft的拥有者用户id
        Nft transferNft = new Nft().
                setUserId(receiveUser.getId()).
                setTransferHash(realHash);
        transferNft.setId(nft.getId());
        updateById(transferNft);
        // 保存nft的转让记录
        NftTransferRecord record = new NftTransferRecord();
        record.setNftId(nft.getId()).
                setNftHash(nft.getNftHash()).
                setTransferHash(realHash).
                setFromAddress(user.getWallet()).
                setToAddress(receiveUser.getWallet());
        nftTransferRecordService.save(record);
        return true;
    }

    private String getCodeType(String type) {
        switch (type) {
            case "sms":
                return smsProperties.getTransferNftMessageCodetype();
            case "email":
                return smsProperties.getTransferNftEmailCodetype();
            case "voice":
                return smsProperties.getTransferNftVoiceCodetype();
            default:
                throw GlobalException.newException(ResultCode.VALIDATE_FAILED, "短信类型【" + type + "】的传参有误,取值范围是{sms,email,voice}");
        }
    }

    @Override
    public List<Nft> list(Integer pageNum, Integer pageSize, Integer categoryId) {
        return nftMapper.list(pageNum, pageSize, categoryId);
    }

    @Override
    public Boolean collection(Integer id) {
        int userId = JwtUtil.getUserIdFromToken(request.getHeader("Authorization"));
        String key = RedisConstant.COLLECTION_USER_PREFIX + userId;
        // 如果用户收藏的nft已经存在列表里,那么就是取消收藏
        if (redisUtil.sIsMember(key, id.toString())) {
            collectionService.deleteByUserIdAndNftId(userId, id);
            return redisUtil.sRemove(key, id.toString()) == 1;
        } else {
            TbCollection collection = new TbCollection().setNftId(id).setUserId(userId);
            try {
                collectionService.save(collection);
            } catch (DuplicateKeyException e) {
                collectionService.deleteByUserIdAndNftId(userId, id);
            }
            return redisUtil.sAdd(key, id.toString()) == 1;
        }
    }


    @Override
    public List<CollectionNftVo> getCollectionList() {
        int userId = JwtUtil.getUserIdFromToken(request.getHeader("Authorization"));
        // 从redis里获取当前用户的收藏列表
        Set<String> set = redisUtil.setMembers(RedisConstant.COLLECTION_USER_PREFIX + userId);
        List<Integer> list;
        if (CollectionUtil.isEmpty(set)) {
            // redis里为空,再从mysql里查一次
            list = collectionService.getNftIdListByUserId(userId);
            if (CollectionUtil.isNotEmpty(list)) {
                // 往redis里保存一遍
                String[] array = new String[list.size()];
                for (int i = 0; i < list.size(); i++) {
                    array[i] = list.get(i).toString();
                }
                redisUtil.sAdd(RedisConstant.COLLECTION_USER_PREFIX + userId, array);
            }
        } else {
            list = set.stream().map(Integer::valueOf).collect(Collectors.toList());
        }
        // mysql和redis里都为空,直接返回空集合
        if (CollectionUtil.isEmpty(list)) {
            return new ArrayList<>();
        }
        return nftMapper.getCollectionList(list);
    }

    @Override
    public List<CollectionNftVo> listCurrent(Integer categoryId, Integer userId) {
        return nftMapper.listCurrent(categoryId, userId);
    }

    @Override
    public String generateNftId(Integer categoryId) {
        Category category = categoryService.getById(categoryId);
        if (category == null) {
            throw GlobalException.newException(ResultCode.DATA_ERROR, "没找到此类目");
        }
        int code = RandomUtil.randomInt(1000, 10000);
        return String.format("INMVO-%s-%s%s", category.getEnglishName(), System.currentTimeMillis(), code);
    }

    @Override
    public PageInfo<NftListVo> page(Integer pageNum, Integer pageSize, Integer categoryId, String name, String theme, String telephone, Integer status, String start, String end) {
        PageHelper.startPage(pageNum, pageSize);
        DateTime startDate = null;
        DateTime endDate = null;
        if (StringUtils.isNotBlank(start)) {
            startDate = DateUtil.parse(start + " 00:00:00", "yyyy-MM-dd HH:mm:ss");
        }
        if (StringUtils.isNotBlank(end)) {
            endDate = DateUtil.parse(end + " 23:59:59", "yyyy-MM-dd HH:mm:ss");
        }
        List<NftListVo> list = nftMapper.page(categoryId, name, telephone, theme, status, startDate, endDate);
        return new PageInfo<>(list);
    }

    @Override
    public Boolean top(Integer id) {
        Nft nft = getById(id);
        // 未上架的nft不能置顶
        if (SystemConstant.BOOLEAN_DATA_FALSE.equals(nft.getStatus())) {
            throw GlobalException.newException(ResultCode.FAILED, "未上架的nft不能置顶");
        }
        nft.setIsTop(SystemConstant.BOOLEAN_DATA_TRUE);
        return updateById(nft);
    }

    @Override
    public Boolean untop(Integer id) {
        Nft nft = new Nft();
        nft.setIsTop(SystemConstant.BOOLEAN_DATA_FALSE);
        nft.setId(id);
        return updateById(nft);
    }

    @Override
    public Boolean onShelf(Integer id) {
        Nft nft = new Nft();
        nft.setStatus(SystemConstant.BOOLEAN_DATA_TRUE);
        nft.setId(id);
        return updateById(nft);
    }

    @Override
    public Boolean offShelf(Integer id) {
        Nft nft = new Nft();
        nft.setStatus(SystemConstant.BOOLEAN_DATA_FALSE);
        nft.setIsTop(SystemConstant.BOOLEAN_DATA_FALSE);
        nft.setId(id);
        return updateById(nft);
    }

    @Override
    public void download(Integer id) {
        Nft nft = getById(id);
        if (nft == null) {
            throw GlobalException.newException(ResultCode.DATA_ERROR, "没找到此nft详情");
        }
        if (SystemConstant.BOOLEAN_DATA_FALSE.equals(nft.getIsArchives()) || SystemConstant.BOOLEAN_DATA_TRUE.equals(nft.getIsGrant())) {
            throw GlobalException.newException(ResultCode.FILE_DOWNLOAD_ERROR, "当前文件不支持下载");
        }
        ossUtil.downloadFile(nft.getFileUrl(), nft.getFileName());
    }

}
