package com.fzm.portal.controller;

import cn.hutool.crypto.SecureUtil;
import com.fzm.common.annotation.Authentication;
import com.fzm.common.constant.RedisConstant;
import com.fzm.common.entity.CommemorateNft;
import com.fzm.common.entity.Label;
import com.fzm.common.entity.User;
import com.fzm.common.entity.vo.CommemorateNftVo;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.model.ResponseModel;
import com.fzm.common.service.CategoryService;
import com.fzm.common.service.CommemorateNftService;
import com.fzm.common.service.LabelService;
import com.fzm.common.service.UserService;
import com.fzm.common.utils.JwtUtil;
import com.fzm.common.utils.OssUtil;
import com.fzm.common.utils.RedisUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.redisson.api.RSemaphore;
import org.redisson.api.RedissonClient;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.List;
import java.util.Locale;

/**
 * @author tangtuo
 * @date 2021/7/1 14:37
 */
@RestController
@RequestMapping(value = "/commemorate/nft")
@Api(tags = "纪念版nft")
public class CommemorateNftController {

    @Resource
    private CommemorateNftService commemorateNftService;

    @Resource
    private OssUtil ossUtil;

    @Resource
    private UserService userService;

    @Resource
    private CategoryService categoryService;

    @Resource
    private RedissonClient redisson;

    @Resource
    private RedisUtil redisUtil;

    @Authentication
    @PostMapping("/publish")
    @ApiOperation(value = "发行纪念版nft", notes = "返回发行成功的数量")
    public ResponseModel<Integer> publish(@ApiParam(value = "作品名称", required = true) String name,
                                          @ApiParam(value = "作品简介", required = true) String synopsis,
                                          @ApiParam(value = "类目id", required = true) Integer categoryId,
                                          @ApiParam(value = "文件", required = true) MultipartFile file,
                                          @ApiParam(value = "封面", required = true) MultipartFile cover,
                                          @ApiParam(value = "发行数量", required = true) Integer count) throws IOException {
        if (file == null || cover == null) {
            throw GlobalException.newException(ResultCode.VALIDATE_FAILED, "文件不能为空");
        }
        String coverName = cover.getOriginalFilename().toLowerCase();
        if (!coverName.endsWith(".png") && !coverName.endsWith(".jpg")) {
            throw GlobalException.newException(ResultCode.FILE_UPLOAD_ERROR, "封面类型有误");
        }
        String fileUrl = ossUtil.putSimpleObject(file);
        String coverUrl = ossUtil.putSimpleObject(cover);
        CommemorateNft commemorateNft = new CommemorateNft().
                setName(name).
                setCover(coverUrl).
                setCount(count).
                setSynopsis(synopsis).
                setFileName(file.getOriginalFilename()).
                setFileUrl(fileUrl).
                setCategoryId(categoryId).
                setFileHash(SecureUtil.md5(file.getInputStream()));
        Integer id = commemorateNftService.publish(commemorateNft);
        return ResponseModel.success(id);
    }

    @Authentication
    @PostMapping("/receive")
    @ApiOperation(value = "领取纪念版nft")
    public ResponseModel<Boolean> receive(@RequestParam Integer id) throws InterruptedException {
        Boolean result = commemorateNftService.receive(id);
        return ResponseModel.success(result);
    }

    //@Authentication
    @GetMapping("/get/{id}")
    @ApiOperation(value = "纪念版nft详情页")
    public ResponseModel<CommemorateNftVo> getById(@PathVariable Integer id, @RequestHeader(required = false) String Authorization) {
        CommemorateNft nft = commemorateNftService.getById(id);
        if (nft == null) {
            throw GlobalException.newException(ResultCode.DATA_ERROR, "此纪念版nft已下架");
        }
        User user = userService.getById(nft.getUserId());
        CommemorateNftVo commemorateNftVo = new CommemorateNftVo(nft, user);
        commemorateNftVo.setCategory(categoryService.getById(nft.getCategoryId()).getCategoryName());
        RSemaphore semaphore = redisson.getSemaphore(RedisConstant.COMMEMORATE_NFT_PREFIX + id);
        int permits = semaphore.availablePermits();
        commemorateNftVo.setReceiveCount(nft.getCount() - permits);
        // 未登录状态
        if (StringUtils.isBlank(Authorization) || "Bearer null".equals(Authorization)) {
            commemorateNftVo.setStatus(1);
        } else {
            // 已登录
            Boolean receive = redisUtil.sIsMember(RedisConstant.COMMEMORATE_NFT_MEMBERS_PREFIX + id, JwtUtil.getUserIdFromToken(Authorization).toString());
            // 当前用户已领取
            if (receive) {
                commemorateNftVo.setStatus(0);
            } else {
                // 未领取完毕
                if (commemorateNftVo.getPublishCount() > commemorateNftVo.getReceiveCount()) {
                    commemorateNftVo.setStatus(1);
                } else {
                    commemorateNftVo.setStatus(2);
                }
            }
        }

        return ResponseModel.success(commemorateNftVo);
    }

    @GetMapping("/list")
    @ApiOperation(value = "首页banner页(纪念版nft列表)")
    public ResponseModel<List<CommemorateNft>> list() {
        return ResponseModel.success(commemorateNftService.list());
    }
}
